@@ -166,9 +166,12 @@ async def _play_game_ai(client, player_id, ai, realtime, step_time_limit, game_t
 
     game_data = await client.get_game_data()
     game_info = await client.get_game_info()
+    ping_response = await client.ping()
 
     # This game_data will become self._game_data in botAI
-    ai._prepare_start(client, player_id, game_info, game_data, realtime=realtime)
+    ai._prepare_start(
+        client, player_id, game_info, game_data, realtime=realtime, base_build=ping_response.ping.base_build
+    )
     state = await client.observation()
     # check game result every time we get the observation
     if client._game_result:
@@ -340,9 +343,13 @@ async def _play_replay(client, ai, realtime=False, player_id=0):
 
     game_data = await client.get_game_data()
     game_info = await client.get_game_info()
+    ping_response = await client.ping()
+
     client.game_step = 1
     # This game_data will become self._game_data in botAI
-    ai._prepare_start(client, player_id, game_info, game_data, realtime=realtime)
+    ai._prepare_start(
+        client, player_id, game_info, game_data, realtime=realtime, base_build=ping_response.ping.base_build
+    )
     state = await client.observation()
     # Check game result every time we get the observation
     if client._game_result:
@@ -482,7 +489,13 @@ async def _host_game(
 
 
 async def _host_game_aiter(
-    map_settings, players, realtime, portconfig=None, save_replay_as=None, step_time_limit=None, game_time_limit=None,
+    map_settings,
+    players,
+    realtime,
+    portconfig=None,
+    save_replay_as=None,
+    step_time_limit=None,
+    game_time_limit=None,
 ):
     assert players, "Can't create a game without players"
 
@@ -519,7 +532,12 @@ def _host_game_iter(*args, **kwargs):
 
 
 async def _join_game(
-    players, realtime, portconfig, save_replay_as=None, step_time_limit=None, game_time_limit=None,
+    players,
+    realtime,
+    portconfig,
+    save_replay_as=None,
+    step_time_limit=None,
+    game_time_limit=None,
 ):
     async with SC2Process(fullscreen=players[1].fullscreen) as server:
         await server.ping()
